package com.serialio.sdm_tcpip_client

import android.app.Application
import android.app.NotificationChannel
import android.app.NotificationManager
import android.content.Context
import android.content.Intent
import android.net.Uri
import android.os.Build
import android.os.PowerManager
import android.provider.Settings
import androidx.core.content.ContextCompat
import com.restock.loggerlib.Logger
import com.restock.loggerlib.LoggerSinglton
import com.serialio.sdm_tcpip_client.service.SdmClientForegroundService
import com.serialio.sdm_tcpip_client.utils.CLIENT_CHANNEL_ID
import com.serialio.sdm_tcpip_client.utils.STARTFOREGROUND_ACTION
import com.serialio.sdm_tcpip_client.utils.STOPFOREGROUND_ACTION
import dagger.hilt.android.HiltAndroidApp

@HiltAndroidApp
class App : Application() {
    companion object {
        val logger: Logger = LoggerSinglton.getInstance()
    }

    override fun onCreate() {
        super.onCreate()
        createNotificationSdmClientNotificationChannel(this)
    }

    private fun createNotificationSdmClientNotificationChannel(ctx: Context) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            val serviceChannel = NotificationChannel(
                CLIENT_CHANNEL_ID,
                getString(R.string.sdm_client_channel),
                NotificationManager.IMPORTANCE_DEFAULT
            )
            serviceChannel.setSound(null, null)
            val manager = ctx.getSystemService(
                NotificationManager::class.java
            )
            manager.createNotificationChannel(serviceChannel)
        }
    }

    fun startSioSdmServerService() {
        val startIntent = Intent(this, SdmClientForegroundService::class.java)
        startIntent.action = STARTFOREGROUND_ACTION
        ContextCompat.startForegroundService(this, startIntent)
    }

    fun stopSioSdmClientService() {
        val stopIntent = Intent(this, SdmClientForegroundService::class.java)
        stopIntent.action = STOPFOREGROUND_ACTION
        startService(stopIntent)
    }

    fun checkBatteryOptimizations()
    {
        logger.putt("checkBatteryOptimizations\n")
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {

            val packageName = packageName
            val pm = getSystemService(POWER_SERVICE) as PowerManager
            if (!pm.isIgnoringBatteryOptimizations(packageName)) {
                logger.putt("checkBatteryOptimizations.IgnoringBatteryOptimizations: FALSE\n")
                Intent().apply {
                    action = Settings.ACTION_REQUEST_IGNORE_BATTERY_OPTIMIZATIONS
                    data = Uri.parse("package:$packageName")
                    flags = Intent.FLAG_ACTIVITY_NEW_TASK
                    startActivity(this)
                }

            }
        }
    }
}
