package com.serialio.sdm_tcpip_client.data.tcp_client

import com.serialio.sdm_tcpip_client.utils.ClientConnectionStatus
import com.serialio.sdm_tcpip_client.data.tcp_client.model.TcpClient
import com.serialio.sdm_tcpip_client.data.tcp_protocol.ConstantsTcp
import com.serialio.sdm_tcpip_client.data.tcp_protocol.parseClientData
import com.serialio.sdm_tcpip_client.domain.repository.TcpClientRepository
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.flow.*
import javax.inject.Inject
import javax.inject.Singleton


@Singleton
class TcpClientRepositoryImpl @Inject constructor(): TcpClientRepository {

    private var mTcpClient: TcpClient? = null

    //val _clientConnectionStatus  = MutableStateFlow<ClientConnectionStatus>(ClientConnectionStatus.Disconnected)
    //val clientConnectionStatus = _clientConnectionStatus.asSharedFlow()

    val _dataSentToServer  = MutableSharedFlow<String>()
    val dataSentToServer: Flow<String> = _dataSentToServer

    override suspend fun connect(ip: String, port: Int) : Flow<ClientConnectionStatus> =
        flow {
        mTcpClient = TcpClient(port, ip)
        mTcpClient?.run()?.collect { connectionStatus ->
                when (connectionStatus) {
                    is ClientConnectionStatus.Connected ->
                        emit(ClientConnectionStatus.Connected)
                    is ClientConnectionStatus.Error ->
                        emit(ClientConnectionStatus.Error(connectionStatus.message ?: "Connection Error"))
                    is ClientConnectionStatus.Connecting ->
                        emit(ClientConnectionStatus.Connecting)
                    is ClientConnectionStatus.Disconnected ->
                        emit(ClientConnectionStatus.Disconnected)
                    is ClientConnectionStatus.OnDataReceived -> {
                        emit(ClientConnectionStatus.OnDataReceived(connectionStatus.message!!))
                    }
                }

            }
        }.flowOn(Dispatchers.IO)

    override suspend fun clientConnectionStatus(): Flow<ClientConnectionStatus> {
        TODO("Not yet implemented")
    }


    override suspend fun sendMsg(msg: String) : Boolean{
        if (mTcpClient == null) {
            return false
        }

        mTcpClient?.sendMessage(msg)
        _dataSentToServer.emit(clientRequestToHuman(msg))
        return true
    }

    override suspend fun stopClient(){
        mTcpClient?.stopClient()
    }

    override suspend fun dataSentToServer(): Flow<String> {
        return dataSentToServer
    }

    private fun clientRequestToHuman(msg: String): String {
        var humanMessage = msg;
        val requestItemTcp = parseClientData(msg)
        when(requestItemTcp?.command){
            ConstantsTcp.CMD_CLIENT_IDENTIFICATION->
                humanMessage = "Identification [password: ${requestItemTcp.parameters?.password}]";
            ConstantsTcp.CMD_GET_REMOTE_DEVICES->
                humanMessage = "Remote devices request";
            ConstantsTcp.CMD_DEVICE_CONNECT->
                humanMessage = "Assign to remote device: ${requestItemTcp.parameters?.connect_disconnect}\nDevice ID: ${requestItemTcp.parameters?.device_id}";
            ConstantsTcp.CMD_DEVICE_COMMAND->
                humanMessage = "Sent Command: '${requestItemTcp.parameters?.command_name}'\nDevice ID: ${requestItemTcp.parameters?.device_id}";
        }
        return  humanMessage
    }

}
