package com.serialio.sdm_tcpip_client.data.tcp_client.model

import android.util.Log
import com.serialio.sdm_tcpip_client.App
import com.serialio.sdm_tcpip_client.utils.ClientConnectionStatus
import io.ktor.network.selector.*
import io.ktor.network.sockets.*
import io.ktor.utils.io.jvm.javaio.*
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.delay
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.flow
import kotlinx.coroutines.flow.flowOn
import java.io.*
import java.util.concurrent.atomic.AtomicBoolean


class TcpClient internal constructor(
    private val port: Int,
    private val ip: String,
) {
    // message to send to the server
    private var mServerMessage: String? = null

    // while this is true, the server will continue running
    private var mRun: AtomicBoolean = AtomicBoolean(false)

    // used to send messages
    private var mBufferOut: PrintWriter? = null

    // used to read messages from the server
    private var mBufferIn: BufferedReader? = null
    var socket: io.ktor.network.sockets.Socket? = null
    var selectorManager: SelectorManager? = null

    fun sendMessage(message: String) {
        val runnable = Runnable {
            if (mBufferOut != null) {
                Log.d(TAG, "Sending: $message")
                App.logger.putt("TcpClient.sendMessage\n${message}\n")
                mBufferOut!!.println(message)
                mBufferOut!!.flush()
            }
        }
        val thread = Thread(runnable)
        thread.start()
    }

    /**
     * Close the connection and release the members
     */
    fun stopClient() {
        if(mBufferIn != null) {
            val sClient = socket?.localAddress?.toJavaAddress().toString()
            Log.d(TAG, "stopClient: ${sClient}")
            App.logger.putt("stopClient: ${sClient}\n")
            mRun = AtomicBoolean(false)
            socket?.close()
            selectorManager?.close()

            if (mBufferOut != null) {
                mBufferOut?.flush()
                mBufferOut?.close()
            }
            mBufferIn = null
            mBufferOut = null
            mServerMessage = null
        }
    }

    fun run(): Flow<ClientConnectionStatus> = flow {
        Log.d(TAG, "run")
            mRun = AtomicBoolean(true)

        try {
            emit(ClientConnectionStatus.Connecting)
            //here you must put your computer's IP address.
            Log.d(TAG, "C: Connecting...")
            App.logger.putt("TcpClient.run(): Connecting: ${ip}\n")

            //create a socket to make the connection with the server
            selectorManager = SelectorManager(Dispatchers.IO)
            socket = aSocket(selectorManager!!).tcp().connect(InetSocketAddress(ip, port)){
                //socketTimeout = 5000
                //this.keepAlive = true
            }

            val receiveChannel = socket?.openReadChannel()
            val sendChannel = socket?.openWriteChannel(autoFlush = true)
            try {
                //sends the message to the server
                mBufferOut = PrintWriter(BufferedWriter(OutputStreamWriter(sendChannel?.toOutputStream())),true)
                //receives the message which the server sends back
                mBufferIn = BufferedReader(InputStreamReader(receiveChannel?.toInputStream()))

                emit(ClientConnectionStatus.Connected)

                while (mRun.get()) {
                    mServerMessage = receiveChannel?.readUTF8Line(Int.MAX_VALUE)
                    if (mServerMessage != null) {
                        Log.d(TAG, "S: Received Message1: '$mServerMessage'")
                        App.logger.putt("TcpClient.run(): Received Message1: $mServerMessage\n")
                        emit(ClientConnectionStatus.OnDataReceived(mServerMessage!!))
                        delay(50)
                    }
                    else
                    {
                        Log.d(TAG, "S: mServerMessage == null")
                        App.logger.putt("TcpClient.run(): mServerMessage == null\n")
                        emit(ClientConnectionStatus.Disconnected)
                        mRun.set(false)
                    }
                }
            }
            catch (e: Exception) {
                if(mBufferIn != null) {
                    Log.d(TAG, "Client disconnects")

                    App.logger.putt("TcpClient.run(): Client disconnects (Exception: ${e.message})\n")
                    App.logger.putt("TcpClient.run(): tackTrace:\n ${e.stackTraceToString()})\n")
                    emit(ClientConnectionStatus.Disconnected)
                }
                else {
                    Log.d(TAG, "Exception1 ${e.message}")
                    App.logger.putt("TcpClient.run(): Exception1 ${e.message}\n")
                    emit(ClientConnectionStatus.Error(e.message ?: "Exception"))
                }
            }
            finally {
                Log.d(TAG, "finally ")
                App.logger.putt("TcpClient.run(): finally 1\n")
            }
        }
        catch (e: Exception) {
            Log.d(TAG, "Exception2 ${e.message}")
            App.logger.putt("TcpClient.run(): Exception2: ${e.message}\n")
            emit(ClientConnectionStatus.Error(e.message ?: "Exception"))
        }
        finally {
            App.logger.putt("TcpClient.run(): finally 2\n")
            stopClient()
        }
    }.flowOn(Dispatchers.IO)

    companion object {
        val TAG = TcpClient::class.java.simpleName
    }
}
