package com.serialio.sdm_tcpip_client.data.upd_client

import android.util.Log
import com.serialio.sdm_tcpip_client.App
import com.serialio.sdm_tcpip_client.data.upd_client.data.DataUdp
import io.ktor.network.selector.*
import io.ktor.network.sockets.*
import io.ktor.utils.io.core.*
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.flow
import kotlinx.coroutines.flow.flowOn
import java.io.IOException
import java.net.DatagramPacket
import java.net.DatagramSocket
import java.net.InetAddress
import java.net.SocketException
import java.util.concurrent.atomic.AtomicBoolean

/**
 * Created by Mykola Blyzniuk on 26.08.2022.
 */
class UdpDataSource internal constructor(
    private val portLocal: Int,
    private val portRemote: Int,
    private val ipLocal: String,
) {

    private var mRun: AtomicBoolean = AtomicBoolean(false)
    var readDatagramSocket: BoundDatagramSocket? = null

    fun sendUdpMessage(ipRemote: String, message: String) {
        val runnable = Runnable {
            if (!ipRemote.isEmpty() && !message.isEmpty()) {
                App.logger.putt("UdpClient.sendUdpMessage[${ipRemote}]:${message}\n")
                var udpSocketSend: DatagramSocket? = null
                try {
                    val serverAddr: InetAddress = InetAddress.getByName(ipRemote)

                    udpSocketSend = DatagramSocket(null)
                    udpSocketSend.broadcast = true
                    udpSocketSend.reuseAddress = true
                    udpSocketSend.connect(serverAddr, portRemote)

                    val buf = message.toByteArray()
                    val packet = DatagramPacket(buf, buf.size, serverAddr, portRemote)
                    //App.logger.putt("UdpClient.sendUdpMessage.send [${ipRemote}]\n${message}\n")
                    udpSocketSend.send(packet)
                }
                catch (e: SocketException) {
                    App.logger.putt("UdpClient.sendUdpMessage.Socket Error: [${e.message.toString()}\n")
                    udpSocketSend?.close()
                }
                catch (e: IOException) {
                    App.logger.putt("UdpClient.sendUdpMessage.IO Error: [${e.message.toString()}\n")
                    udpSocketSend?.close()
                }
                finally {
                    App.logger.putt("UdpClient.sendUdpDatatgram.Socket close()\n")
                    udpSocketSend?.close()
                }
            }
        }
        val thread = Thread(runnable)
        thread.start()
    }

    suspend fun listenUDP(): Flow<DataUdp> = flow {
        App.logger.putt("UdpClient.listenUDP.START: ${portLocal}\n")
        mRun = AtomicBoolean(true)

        emit(DataUdp(ip = "", message = "START"))

        readDatagramSocket = aSocket(SelectorManager(Dispatchers.IO)).udp()
            .bind(InetSocketAddress("0.0.0.0", portLocal)) {
                this.broadcast = true
                this.reuseAddress = true
            }

        readDatagramSocket.use { udpSocket ->
            try {
                while (mRun.get()) {
                    val datagram = udpSocket?.receive()
                    Log.d(TAG, "received")
                    val line = datagram?.packet?.readText() ?: ""

                    Log.d(TAG, datagram?.address?.toJavaAddress().toString())
                    var remoteAddr = datagram?.address?.toJavaAddress().toString()
                    App.logger.putt("UdpClient.listenUDP.received [${remoteAddr}]: ${line} \n")

                    val pos1 = remoteAddr.indexOf("/")
                    val pos2 = remoteAddr.indexOf(":")
                    if (pos1 >= 0 && pos2 >= 0 && pos2 > pos1) {
                        val remoteAddr1 = remoteAddr.substring(remoteAddr.indexOf("/") + 1,
                            remoteAddr.indexOf(":"))
                        remoteAddr = remoteAddr1
                    }
                    Log.d(TAG, remoteAddr)
                    App.logger.putt("UdpClient.listenUDP.addr: ${remoteAddr}\n")

                    emit(DataUdp(ip = remoteAddr, message = line))
                }
                App.logger.putt("UdpClient.listenUDP.while closed\n")
                mRun.set(false)
            }
            catch (e: Throwable) {
                App.logger.putt("UdpClient.listenUDP.Throwable: ${e.message.toString()} \n")
                udpSocket?.close()
                mRun.set(false)
            }
        }
        App.logger.putt("UdpClient.listenUDP.STOP: ${portLocal}\n")
        emit(DataUdp(ip = "", message = "STOP"))
    }.flowOn(Dispatchers.IO)

    fun stopUdpListen() {
        App.logger.putt("UdpClient.stopUdpListen\n")
        mRun = AtomicBoolean(false)
        readDatagramSocket?.close()
    }

    companion object {
        val TAG = UdpDataSource::class.java.simpleName
    }

}
