package com.serialio.sdm_tcpip_client.domain

import com.serialio.sdm_tcpip_client.App
import com.serialio.sdm_tcpip_client.data.tcp_protocol.ConstantsTcp
import com.serialio.sdm_tcpip_client.data.tcp_protocol.model_response_dto.RemoteDeviceResponseTcp
import com.serialio.sdm_tcpip_client.data.tcp_protocol.model_response_dto.ResponseItemTcp
import com.serialio.sdm_tcpip_client.domain.repository.SdmDeviceRepository
import com.serialio.sdm_tcpip_client.utils.PeripheralConnectionStatus
import com.serialio.sdm_tcpip_client.utils.ServerResponse
import dagger.hilt.android.scopes.ViewModelScoped
import kotlinx.coroutines.flow.MutableStateFlow
import javax.inject.Inject

/**
 * Created by Mykola Blyzniuk on 15.07.2022.
 */
@ViewModelScoped
class ServerResponseEngine @Inject constructor(
    val sdmDeviceRepository: SdmDeviceRepository
) {
    val serverResponseFlow  = MutableStateFlow<ServerResponse>(ServerResponse.ClientIdentification())
    val peripheralConnectionStatusFlow  = MutableStateFlow(PeripheralConnectionStatus())

    fun parseServerData(response: ResponseItemTcp) {
        App.logger.putt("parseServerData ${response}\n")

        when (response.command) {
            //--------------------------------------------------------------------------------------
            ConstantsTcp.CMD_CLIENT_IDENTIFICATION -> {
               if(response.status == ConstantsTcp.STATUS_PASSED)
                    serverResponseFlow.value = ServerResponse.ClientIdentification(true, "Identification ${ConstantsTcp.STATUS_PASSED}")
                else
                   serverResponseFlow.value = ServerResponse.ClientIdentification(false, "Identification ${ConstantsTcp.STATUS_FAILED}")
            }
            //--------------------------------------------------------------------------------------
            ConstantsTcp.CMD_GET_REMOTE_DEVICES -> {
                // Update device list from Server
                if(response.status == ConstantsTcp.STATUS_PASSED) {
                    val params = response.parameters
                    params?.remote_devices?.let {
                            val deviceListInfo = makeDeviceListInfo(it)
                            serverResponseFlow.value = ServerResponse.PeripheralDeviceList(
                                isSuccess = true,
                                deviceListInfo,
                                params)
                        }
                }
                else
                    serverResponseFlow.value = ServerResponse.PeripheralDeviceList(isSuccess = false, "Peripheral Devices request FAILED")
            }
            //--------------------------------------------------------------------------------------
            ConstantsTcp.CMD_DEVICE_CONNECT -> {
                // remote device status
                val params = response.parameters
                val deviceId = response.parameters?.device_id
                if(response.status == ConstantsTcp.STATUS_PASSED) {
                    // Update UI: show state of the device
                    val state = response.parameters?.device_state
                    peripheralConnectionStatusFlow.value = PeripheralConnectionStatus(deviceId, state)
                    serverResponseFlow.value = ServerResponse.PeripheralDeviceStatus(isSuccess = true, message = "Device ID: $deviceId\nStatus: $state", params)
                }else{
                    serverResponseFlow.value = ServerResponse.PeripheralDeviceStatus(isSuccess = false, message = "Device ID: ${deviceId}\n${ConstantsTcp.DEVICE_STATE_UNAVAILABLE}", params)
                }
            }
            //--------------------------------------------------------------------------------------
            ConstantsTcp.CMD_DEVICE_COMMAND -> {
                // Processing device command response
                var deviceId = response.parameters?.device_id
                val commandName = response.parameters?.command_name
                if(response.status == ConstantsTcp.STATUS_PASSED) {
                    val commandResult = response.parameters?.command_result
                    serverResponseFlow.value = ServerResponse.PeripheralDeviceCmdResponse(isSuccess = true, message = "Device ID: ${deviceId}\nCmd: '$commandName', Result: $commandResult" )
                }else{
                    serverResponseFlow.value = ServerResponse.PeripheralDeviceCmdResponse(isSuccess = false, message = "Device ID: ${deviceId}\nCmd: '$commandName', Result:No response on cmd" )
                }
            }
        }
    }

    private fun makeDeviceListInfo(it: MutableList<RemoteDeviceResponseTcp>): String {
        var deviceListInfo = "Remote devices: ${it.size}"
        var ind = 0
        it.forEach{
            deviceListInfo += "\n${++ind}. id:${it.device_id}\n${it.device_name} [${it.device_type}] State:${it.device_state}\ncommands:"
            it.device_commands?.forEach{
                deviceListInfo += " '${it.cmd_name}'"
            }
        }
        //App.logger.putt("DeviceListInfo:\n ${deviceListInfo}\n")
        return deviceListInfo
    }

}
