package com.serialio.sdm_tcpip_client.domain

import com.serialio.sdm_tcpip_client.data.upd_client.data.ConstantsUdp
import com.serialio.sdm_tcpip_client.data.upd_client.data.DataUdp
import com.serialio.sdm_tcpip_client.domain.repository.UdpRepository
import kotlinx.coroutines.channels.TickerMode
import kotlinx.coroutines.channels.ticker
import kotlinx.coroutines.flow.Flow
import java.util.concurrent.atomic.AtomicBoolean
import javax.inject.Inject
import javax.inject.Singleton

/**
 * Created by Mykola Blyzniuk on 29.08.2022.
 */
@Singleton
class UdpEngine @Inject constructor(
    val udpRepository: UdpRepository
) {
    var isWork = AtomicBoolean(true)

    suspend fun startUdpRequestsTimer(ip: String) {
        val ticker = ticker(1000, 1000, mode = TickerMode.FIXED_PERIOD)
        var cnt = 0
        for (event in ticker) {
            if (!isWork.get() || cnt++ > 4) break
            udpRepository.sendUdpData(ip.ipMaskUpdate(), ConstantsUdp.UDP_CLIENT_REQUEST) //"192.168.1.255"
        }
        ticker.cancel()
    }

    fun stopTimer() {
        isWork.set(false)
    }

    suspend fun stopUpdListening() {
        udpRepository.stopUdpListening()
    }

    fun listenUdpData(portLocal: Int, portRemote: Int, ipLocal: String): Flow<DataUdp> {
        isWork = AtomicBoolean(true)
        return udpRepository.listenUdpData(portLocal, portRemote, ipLocal)
    }

    fun String.ipMaskUpdate(): String{
        return this.splitToSequence(".").take(3).joinToString (  separator = "." , postfix = ".255" )
    }

}
