package com.serialio.sdm_tcpip_client.presentation.device_manager

import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.cardview.widget.CardView
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.ListAdapter
import androidx.recyclerview.widget.RecyclerView
import com.serialio.sdm_tcpip_client.App
import com.serialio.sdm_tcpip_client.R
import com.serialio.sdm_tcpip_client.data.repository.sdm_device.model.SdmDevice

class DeviceManagerAdapter(private val context : Context, private val onClick: (SdmDevice) -> Unit, private val onClientClick: (SdmDevice) -> Unit) :
    ListAdapter<SdmDevice, DeviceManagerAdapter.SdmDevicesViewHolder>(DevicesDiffCallback){
    var selectedItem: Int = -1
    var selectedDevice: SdmDevice? = null

    fun submitList(list: List<SdmDevice>, name: String) {
        super.submitList(list)
    }

    fun updateSelectedItem(id: String) {
        selectedDevice = currentList.first { it.deviceId == id }
        selectedItem = currentList.indexOfFirst { it.deviceId == id }
        notifyDataSetChanged()
    }

    inner class SdmDevicesViewHolder(val context: Context, itemView: View, val onClick: (SdmDevice) -> Unit, val onClientClick: (SdmDevice) -> Unit) :
        RecyclerView.ViewHolder(itemView) {
        private val tvName: TextView = itemView.findViewById(R.id.tvName)
        private val tvType: TextView = itemView.findViewById(R.id.tvType)
        private val tvDeviceId: TextView = itemView.findViewById(R.id.tvDeviceId)
        private val tvDeviceStatus: TextView = itemView.findViewById(R.id.tvDeviceStatus)
        private val cardView: CardView = itemView.findViewById(R.id.card)
        private var currentDevice: SdmDevice? = null

        fun bind(device: SdmDevice,onItemClick: ((SdmDevice) -> Unit)?, position: Int) {
            App.logger.putt("DeviceManagerAdapter.bind: ${device}")
            currentDevice = device
            when {
                selectedItem == position -> {
                    cardView.setCardBackgroundColor(itemView.context.resources.getColor(R.color.teal_200, null))
                }
                else -> {
                    cardView.setCardBackgroundColor(itemView.context.resources.getColor(R.color.white, null))
                }
            }
            if (selectedItem != position) {
                itemView.setOnClickListener { onItemClick?.invoke(currentDevice!!) }
            }
            tvName.text = device.name
            tvType.text = device.type
            tvDeviceId.text = device.deviceId
            tvDeviceStatus.text = device.status
        }
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): SdmDevicesViewHolder {
        val view = LayoutInflater.from(parent.context)
            .inflate(R.layout.item_sdm_device, parent, false)
        return SdmDevicesViewHolder(context, view, onClick, onClientClick)
    }

    override fun onBindViewHolder(holder: SdmDevicesViewHolder, position: Int) {
        val flower = getItem(position)
        holder.bind(flower, onClick, position)
    }
}

object DevicesDiffCallback : DiffUtil.ItemCallback<SdmDevice>() {
    override fun areItemsTheSame(oldItem: SdmDevice, newItem: SdmDevice): Boolean {
        return oldItem == newItem
    }

    override fun areContentsTheSame(oldItem: SdmDevice, newItem: SdmDevice): Boolean {
        return oldItem.deviceId == newItem.deviceId
    }
}
