package  com.serialio.sdm_tcpip_client.presentation.device_manager

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Toast
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import com.serialio.sdm_tcpip_client.App
import com.serialio.sdm_tcpip_client.data.repository.sdm_device.model.SdmDevice
import com.serialio.sdm_tcpip_client.databinding.FragmentDeviceListBinding
import dagger.hilt.android.AndroidEntryPoint

@AndroidEntryPoint
class DeviceMangerFragment : Fragment() {

    private val deviceManagerViewModel: DeviceMangerViewModel by viewModels()
    private var _binding: FragmentDeviceListBinding? = null
    private val binding get() = _binding!!
     lateinit var devicesAdapter : DeviceManagerAdapter
    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        _binding = FragmentDeviceListBinding.inflate(inflater, container, false)
        val root: View = binding.root
         devicesAdapter =
            DeviceManagerAdapter(requireActivity(), { device -> adapterOnClick(device) },
                { device -> adapterOnClientClick(device) })

        binding.recyclerView.adapter = devicesAdapter
        binding.recyclerView.itemAnimator = null

        deviceManagerViewModel.sdmDevicesLiveData.observe(viewLifecycleOwner) {
            it?.let {
                devicesAdapter.submitList(it.stream().toList())
                devicesAdapter.notifyDataSetChanged()
                if(it.isNullOrEmpty()){
                    binding.selectDevice.visibility = View.GONE
                }else{
                    binding.selectDevice.visibility = View.VISIBLE
                }
            }
        }

        binding.selectDevice.setOnClickListener{
            if(devicesAdapter.selectedDevice == null){
                Toast.makeText(requireContext(), "No device selected", Toast.LENGTH_LONG)
                    .show()
            }else {
                devicesAdapter.selectedDevice?.let {
                    val bundle = Bundle()
                    bundle.putString("device_id", it.deviceId)
                    requireActivity().supportFragmentManager.setFragmentResult("selected_device", bundle)
                }
            }
        }
        return root
    }

    private fun adapterOnClick(device: SdmDevice) {
            devicesAdapter.updateSelectedItem(device.deviceId)
            App.logger.putt("adapterOnClick ${device.deviceId}\n")
    }

    private fun adapterOnClientClick(device: SdmDevice) {
        Toast.makeText(requireContext(), "Client click ${device.name}", Toast.LENGTH_LONG)
            .show()
        App.logger.putt("adapterOnClientClick. Client click ${device.name}\n")
       devicesAdapter.updateSelectedItem(device.name)
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }
}
