package com.serialio.sdm_tcpip_client.presentation.main

import android.app.AlertDialog
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.graphics.Color
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.text.Html
import android.text.TextUtils
import android.view.Menu
import android.view.MenuInflater
import android.view.MenuItem
import android.view.View
import android.widget.*
import androidx.activity.viewModels
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.res.ResourcesCompat
import androidx.localbroadcastmanager.content.LocalBroadcastManager
import androidx.preference.PreferenceManager
import com.serialio.sdm_tcpip_client.*
import com.serialio.sdm_tcpip_client.data.repository.sdm_device.model.SdmDevice
import com.serialio.sdm_tcpip_client.data.tcp_protocol.ConstantsTcp
import com.serialio.sdm_tcpip_client.utils.ClientConnectionStatus
import com.serialio.sdm_tcpip_client.utils.ServerResponse
import com.serialio.sdm_tcpip_client.utils.activateLogging
import com.serialio.sdm_tcpip_client.utils.getIpv4HostAddress
import com.serialio.sdm_tcpip_client.utils.isIpValid
import com.serialio.sdm_tcpip_client.utils.uploadLogFile
import com.serialio.sdm_tcpip_client.presentation.device_manager.DeviceMangerFragment
import dagger.hilt.android.AndroidEntryPoint
import java.io.File
import java.util.*


@AndroidEntryPoint
class MainActivity() : AppCompatActivity() {

    lateinit var LOG_FULLPATH: String
    private var pressedTime: Long = 0

    val handler = Handler(Looper.getMainLooper())
    private val mainActivityViewModel: MainActivityViewModel by viewModels()
    var tv_pheripheral: TextView? = null
    var etMessage: EditText? = null
    var etIP: EditText? = null
    var etPort: EditText? = null
    var btnSend: Button? = null
    var btnServerDevice: Button? = null
    var btnConnect: Button? = null
    var logText: TextView? = null
    var tvClientID: TextView? = null
    var builder = StringBuilder()
    var scrollLog: ScrollView? = null
    private var isConnected: Boolean = false
    private lateinit var clientId: String

    var mLocalBroadcastManager: LocalBroadcastManager? = null
    var mBroadcastReceiver: BroadcastReceiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context?, intent: Intent) {
            if (intent.action == "com.serialio.closeapp") {
                finishAndRemoveTask();
            }
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)
        updateActionBarStatus()

        LOG_FULLPATH = getExternalFilesDir(null).toString() + File.separator + "ClientLogFile.txt"
        activateLogging(LOG_FULLPATH)
        (application as App).startSioSdmServerService()


        mLocalBroadcastManager = LocalBroadcastManager.getInstance(this)
        val mIntentFilter = IntentFilter()
        mIntentFilter.addAction("com.serialio.closeapp")
        mLocalBroadcastManager!!.registerReceiver(mBroadcastReceiver, mIntentFilter)


        tvClientID = findViewById(R.id.tv_client_id)

        //Create unique ClientId ot the first launch of app
        clientId = getClientId().toString()
        if (clientId.isEmpty()) {
            val sClientId = UUID.randomUUID().toString()
            saveClientId(sClientId)
            clientId = sClientId
        }
        mainActivityViewModel.clientID = clientId

        tvClientID?.text = ("ClientID: $clientId")
        etIP = findViewById(R.id.etIP)
        etIP?.setText(iP)
        etPort = findViewById(R.id.etPort)
        etPort?.setText(port)
        val currentip = findViewById<TextView>(R.id.currentip)
        currentip.text = "My IP: " + getIpv4HostAddress()
        etMessage = findViewById(R.id.etMessage)
        tv_pheripheral = findViewById(R.id.tv_pheripheral)
        etMessage?.setText(lastCmd)
        btnSend = findViewById(R.id.btnSend)
        btnServerDevice = findViewById(R.id.btnServerDevice)
        logText = findViewById(R.id.logText)

        scrollLog = findViewById(R.id.scrollLog)
        btnConnect = findViewById<Button>(R.id.btnConnect)
        btnConnect?.setOnClickListener {
            App.logger.putt("btnConnect: clicked\n")
            if (!TextUtils.isEmpty(etIP?.text.toString())) {
                if (!TextUtils.isEmpty(etPort?.text.toString())) {
                    if (isIpValid(etIP?.text.toString())) {
                        try {
                            if (isConnected) {
                                isConnected = false
                                mainActivityViewModel.disconnectServer();
                            } else {
                                val portInt = etPort?.text.toString().toInt()
                                saveIP(etIP?.text.toString())
                                savePort(portInt.toString())
                                mainActivityViewModel.connectServer(
                                    etIP?.text.toString(),
                                    portInt
                                )
                            }
                        }
                        catch (ex: NumberFormatException) {
                            writeToAppLog(
                                "Port not valid: " + etPort?.text.toString(),
                                Type.ERROR
                            )
                            Toast.makeText(
                                this@MainActivity,
                                "Port not valid: " + etPort?.text.toString(),
                                Toast.LENGTH_LONG
                            ).show()
                        }
                    } else {
                        writeToAppLog("IP not valid: " + etIP?.text.toString(), Type.ERROR)
                        Toast.makeText(
                            this@MainActivity,
                            "IP not valid: " + etIP?.text.toString(),
                            Toast.LENGTH_LONG
                        ).show()
                    }
                } else {
                    writeToAppLog("Empty port to connect", Type.ERROR)
                    Toast.makeText(this@MainActivity, "Empty port to connect", Toast.LENGTH_SHORT)
                        .show()
                }
            } else {
                writeToAppLog("Empty IP to connect", Type.ERROR)
                Toast.makeText(this@MainActivity, "Empty IP to connect", Toast.LENGTH_SHORT).show()
            }
        }
        btnServerDevice?.setOnClickListener { v: View? ->
            App.logger.putt("btnServerDevice: clicked\n")
            supportFragmentManager.beginTransaction()
                .add(R.id.frame, DeviceMangerFragment(), "DeviceMangerFragment").commit();
            supportFragmentManager.setFragmentResultListener("selected_device", this) { _, bundle ->
                val fr = supportFragmentManager.findFragmentByTag("DeviceMangerFragment");
                if (fr != null) {
                    supportFragmentManager.beginTransaction().remove(fr).commit()
                }
                val deviceiD = bundle.get("device_id")
                deviceiD?.let {
                    if (!mainActivityViewModel.sdmDevicesLiveData.value.isNullOrEmpty()) {
                        val device =
                            mainActivityViewModel.sdmDevicesLiveData.value?.find { it.deviceId == deviceiD }
                        device?.let {
                            mainActivityViewModel.updateSelectedDevice(device)
                            mainActivityViewModel.sendConnectDeviceRequest(device.deviceId)
                            showSelectCommand(device);
                            saveSelectedDeviceId(device.deviceId)
                        }
                    }
                }
            }
        }
        btnSend?.setOnClickListener { v: View? ->
            App.logger.putt("btnSend: clicked\n")
            if (!TextUtils.isEmpty(etMessage?.text.toString())) {
                saveLastCmd(etMessage?.text.toString())
                mainActivityViewModel.sendCmdToSelectedDevice(etMessage?.text.toString())
            } else {
                writeToAppLog("Empty msg to send", Type.ERROR)
                Toast.makeText(this@MainActivity, "Empty msg to send", Toast.LENGTH_SHORT).show()
            }
        }

        mainActivityViewModel.serverConnectionStatusLiveData.observe(
            this
        ) { connectionStatus ->
            when (connectionStatus) {
                is ClientConnectionStatus.Connecting -> {
                    isConnected = false
                    btnSend?.isEnabled = false

                    btnConnect?.text = "Connect to Server"
                    writeToAppLog("Connecting...", Type.INFO)
                }
                is ClientConnectionStatus.Connected -> {
                    btnSend?.isEnabled = false
                    isConnected = true
                    //btnServerDevice?.isEnabled = true
                    btnConnect?.text = "Connected to ${iP}"
                    writeToAppLog("Connected to ${iP}", Type.INFO)
                    Toast.makeText(this@MainActivity, "Connected", Toast.LENGTH_SHORT).show()

                    (applicationContext as App).checkBatteryOptimizations()
                }
                is ClientConnectionStatus.Disconnected -> {
                    isConnected = false
                    btnSend?.isEnabled = false
                    btnConnect?.text = "Connect to Server"

                    mainActivityViewModel.sdmDevicesLiveData.value?.forEach() {
                        it.status = ""
                    }
                    tv_pheripheral?.setTextColor(Color.GRAY)

                    writeToAppLog("Disconnected from ${iP}", Type.INFO)
                    Toast.makeText(this@MainActivity, "Disconnected", Toast.LENGTH_SHORT).show()
                }
                is ClientConnectionStatus.OnDataReceived -> {
                    if (!connectionStatus.message.isNullOrEmpty()) {
                        App.logger.putt("received message:\n${connectionStatus.message}\n")
                    }
                }
                is ClientConnectionStatus.Error -> {
                    isConnected = false
                    btnSend?.isEnabled = false
                    btnServerDevice?.isEnabled = false
                    btnConnect?.text = "Connect to Server"
                    var sText: String = "Connection: "
                    sText += connectionStatus.message ?: "Failed to connect"
                    writeToAppLog(sText, Type.ERROR)
                    Toast.makeText(this@MainActivity, sText, Toast.LENGTH_SHORT).show()
                }
            }
            buttonStatesToLog("serverConnectionStatusLiveData.ClientConnectionStatus")
        }

        mainActivityViewModel.serverResponseLiveData.observe(
            this
        ) { serverResponse ->
            when (serverResponse) {
                is ServerResponse.ClientIdentification -> {
                    if (!serverResponse.message.isNullOrEmpty())
                        writeToAppLog("< ${serverResponse.message}", Type.RECEIVED)
                }
                is ServerResponse.PeripheralDeviceList -> {
                    if (!serverResponse.message.isNullOrEmpty()) {
                        writeToAppLog("< ${serverResponse.message}", Type.RECEIVED)
                    }
                }
                is ServerResponse.PeripheralDeviceStatus -> {
                    buttonStatesToLog("serverResponseLiveData.ServerResponse.PeripheralDeviceStatus")
                    writeToAppLog("< ${serverResponse.message}", Type.RECEIVED)
                }
                is ServerResponse.PeripheralDeviceCmdResponse -> {
                    writeToAppLog("< ${serverResponse.message}", Type.RECEIVED)
                }
            }
        }


        mainActivityViewModel.dataSentToServerLiveData.observe(
            this
        ) { dataSent ->
            if (!dataSent.isNullOrEmpty())
                writeToAppLog("> ${dataSent}", Type.SENT)
        }

        mainActivityViewModel.peripheralConnectionStatusDeviceLiveData.observe(
            this
        ) { peripheral ->
            if (!peripheral.status.isNullOrEmpty()) {
                App.logger.putt("peripheralConnectionStatusDeviceLiveData.Status ${peripheral.status}\n")
                if (peripheral.status == "Device not connected!" || peripheral.status == "No selected device!") {
                    Toast.makeText(this, peripheral.status, Toast.LENGTH_SHORT).show()
                }
            }
        }

        setSelectedDeviceObserver()

        mainActivityViewModel.sdmDevicesLiveData.observe(this)
        {
            val bHaveList = !it.isEmpty()
            val bEnableSelect = bHaveList && isConnected

            if (btnServerDevice?.isEnabled != bEnableSelect) {
                btnServerDevice?.isEnabled = bEnableSelect
                buttonStatesToLog("sdmDevicesLiveData.SELECT updated")
            }

            // Update selected device on launch
            if (mainActivityViewModel.sdmSelectedDevice.value == null) {
                if (!mainActivityViewModel.sdmDevicesLiveData.value.isNullOrEmpty()) {
                    val device =
                        mainActivityViewModel.sdmDevicesLiveData.value?.find { it.deviceId == getSelectedDeviceId() }
                    device?.let {
                        device.status = ""
                        mainActivityViewModel.updateSelectedDevice(device)
                    }
                }
            }
        }

        mainActivityViewModel.serverIP.observe(
            this
        ) { ip ->
            etIP?.setText(ip)
        }

        buttonStatesToLog("START APP")
    }

    fun setSelectedDeviceObserver() {
        mainActivityViewModel.sdmSelectedDevice.observe(this) {
            buttonStatesToLog("Selected device[mainActivityViewModel._sdmDeviceWorkingWith]:\n${it}")
            tv_pheripheral?.text = "${it.name} [${it.status}]\n${it.deviceId}"
            if (!it.status.isEmpty()) {
                if (it.status == ConstantsTcp.DEVICE_STATE_READY || it.status == ConstantsTcp.DEVICE_STATE_CONNECTED) {
                    tv_pheripheral?.setTextColor(
                        ResourcesCompat.getColor(
                            getResources(),
                            R.color.colorPrimary,
                            null
                        )
                    )
                    btnSend?.isEnabled = true
                } else {
                    tv_pheripheral?.setTextColor(Color.GRAY)
                    btnSend?.isEnabled = false
                }
            }
            buttonStatesToLog("mainActivityViewModel._sdmDeviceWorkingWith")
        }
    }

    private fun updateActionBarStatus() {
        val actionBar = supportActionBar
        if (actionBar != null) {
            actionBar.setTitle(getString(R.string.app_name))
            actionBar.setSubtitle(Html.fromHtml("<font color=\"yellow\"> Ver: " + BuildConfig.VERSION_NAME + "</font>"))
        }
    }

    private fun showSelectCommand(device: SdmDevice) {

        if (device.listCmds.isNullOrEmpty()) {
            Toast.makeText(
                this,
                "Device doesn't have command list! Please select template in SdmServerApp",
                Toast.LENGTH_LONG
            ).show()
            App.logger.putt("showSelectCommand.Device doesn't have command list! Please select template in SdmServerApp\n")
            return;
        }
        val alertDialog: AlertDialog.Builder = AlertDialog.Builder(this)
        alertDialog.setTitle(getString(R.string.select_cmd_to_send))
        val listCmd: List<String> =
            device.listCmds!!.map { "${it.cmd_name} [${it.cmd_description}]" }
        val adapter = ArrayAdapter(this, android.R.layout.simple_list_item_single_choice, listCmd)
        var checkedItem = 0
        alertDialog.setSingleChoiceItems(
            adapter, checkedItem
        ) { _, which ->
            checkedItem = which
        }
        alertDialog.setPositiveButton(getString(R.string.set)) { _, which ->

            val cmd = device.listCmds?.get(checkedItem)?.cmd_name //listCmd.get(checkedItem)
            //mainActivityViewModel.sendCmd(device.deviceId, cmd)
            etMessage?.setText(cmd)
        }
        val alert: AlertDialog = alertDialog.create()
        alert.setCanceledOnTouchOutside(false)
        alert.show()
    }

    override fun onBackPressed() {
        val fr = supportFragmentManager.findFragmentByTag("DeviceMangerFragment");
        if (fr != null) {
            supportFragmentManager.beginTransaction().remove(fr).commit()
            return
        }
        if (pressedTime + 2000 > System.currentTimeMillis()) {
            (application as App).stopSioSdmClientService()
            finishAndRemoveTask();
        } else {
            Toast.makeText(getBaseContext(), "Press back again to exit", Toast.LENGTH_SHORT).show();
            App.logger.putt("onBackPressed.Toast.Press back again to exit\n")
            pressedTime = System.currentTimeMillis()
            return
        }
        super.onBackPressed()
    }


    internal enum class Type {
        INFO, ERROR, SENT, RECEIVED
    }

    private fun writeToAppLog(textIn: String, type: Type) {
        App.logger.putt("n $type\n$textIn\n")
        val text = textIn.replace("\n", "<br />")
        val clean = text.replace("\\P{Print}".toRegex(), "")
        when (type) {
            Type.INFO -> builder.append("<font color=#000000>$clean</font><br>")
            Type.ERROR -> builder.append("<font color=#ff2400>$clean</font><br>")
            Type.SENT -> builder.append("<font color=#00b300>$clean</font><br>")
            Type.RECEIVED -> builder.append("<font color=#410200>$clean</font><br>")
        }

        runOnUiThread {
            logText!!.text = Html.fromHtml(builder.toString())
            scrollLog?.postDelayed({ scrollLog?.fullScroll(View.FOCUS_DOWN) }, 50)
        }
    }

    fun buttonStatesToLog(event: String) {
        App.logger.putt("Button States [${event}]:\nCONNECT:${btnConnect?.isEnabled}, SELECT:${btnServerDevice?.isEnabled}, SEND:${btnSend?.isEnabled}\n")
    }

    override fun onCreateOptionsMenu(menu: Menu): Boolean {
        val inflater: MenuInflater = menuInflater
        inflater.inflate(R.menu.menu, menu)
        return true
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        return when (item.itemId) {
            R.id.send_log -> {
                App.logger.putt("onOptionsItemSelected\n")
                uploadLogFile(this, LOG_FULLPATH, clientId) { result ->
                    handler.post {
                        if (result.contains("Success")) {
                            showSuccessLogUploadAlert(
                                "Log uploaded",
                                "Do you want to clear log file?"
                            )
                        } else {
                            showFailedSendLogAlert("Error", result)
                        }

                    }
                }
                true
            }
            R.id.clear_log -> {
                builder.clear()
                logText!!.text = ""
                true
            }
            else -> super.onOptionsItemSelected(item)
        }
    }

    private fun showSuccessLogUploadAlert(title: String, msg: String) {
        var alert: AlertDialog? = null
        if (alert != null && alert.isShowing()) return;
        val dialogBuilder = this.let { AlertDialog.Builder(it) }
        dialogBuilder.setMessage(msg)
            // if the dialog is cancelable
            ?.setCancelable(false)
            ?.setPositiveButton("Yes") { dialog, id ->
                val logFile = File(LOG_FULLPATH)
                if (logFile.exists()) {
                    logFile.delete()

                    App.logger.open(LOG_FULLPATH, true)
                }
            }
            ?.setNegativeButton("No") { dialog, id ->
                dialog.dismiss()
            }
        alert = dialogBuilder.create()
        alert?.setTitle(title)
        alert?.show()
    }

    private fun showFailedSendLogAlert(title: String, msg: String) {
        var alert: AlertDialog? = null
        if (alert != null && alert.isShowing()) return;
        val dialogBuilder = this.let { AlertDialog.Builder(it) }
        dialogBuilder.setMessage(msg)
            ?.setCancelable(false)
            ?.setPositiveButton("Ok") { dialog, _ ->
                dialog.dismiss()
            }
        alert = dialogBuilder.create()
        alert?.setTitle(title)
        alert?.show()
    }

    val port: String?
        get() = PreferenceManager.getDefaultSharedPreferences(this).getString("port", "")

    fun savePort(port: String?) {
        PreferenceManager.getDefaultSharedPreferences(this).edit().putString("port", port).apply()
    }

    val iP: String?
        get() = PreferenceManager.getDefaultSharedPreferences(this).getString("ip", "")

    fun saveIP(ip: String?) {
        PreferenceManager.getDefaultSharedPreferences(this).edit().putString("ip", ip).apply()
    }

    val lastCmd: String?
        get() = PreferenceManager.getDefaultSharedPreferences(this).getString("cmd", "")

    fun saveLastCmd(cmd: String?) {
        PreferenceManager.getDefaultSharedPreferences(this).edit().putString("cmd", cmd).apply()
    }

    fun getClientId(): String? {
        return PreferenceManager.getDefaultSharedPreferences(this).getString("client_id", "")
    }

    fun saveClientId(client_id: String?) {
        PreferenceManager.getDefaultSharedPreferences(this).edit().putString("client_id", client_id)
            .apply()
    }

    fun getSelectedDeviceId(): String? {
        return PreferenceManager.getDefaultSharedPreferences(this).getString("selectedDeviceId", "")
    }

    fun saveSelectedDeviceId(selectedDeviceId: String?) {
        PreferenceManager.getDefaultSharedPreferences(this).edit()
            .putString("selectedDeviceId", selectedDeviceId)
            .apply()
    }

}
