package com.serialio.sdm_tcpip_client.service

import android.app.Service
import android.content.Intent
import android.os.Binder
import android.os.IBinder
import android.util.Log
import androidx.core.app.NotificationCompat
import androidx.localbroadcastmanager.content.LocalBroadcastManager
import com.serialio.sdm_tcpip_client.App
import com.serialio.sdm_tcpip_client.utils.STARTFOREGROUND_ACTION
import com.serialio.sdm_tcpip_client.utils.STOPFOREGROUND_ACTION
import com.serialio.sdm_tcpip_client.utils.createForegroundNotificationBuilder
import dagger.hilt.android.AndroidEntryPoint

/**
 * Created by Mykola Blyzniuk on 27.07.2022.
 */
@AndroidEntryPoint
class SdmClientForegroundService : Service(){

    var mBinder: IBinder = LocalBinder()

    private val sTAG = "ServiceSdmClient"
    private var notificationBuilder:  NotificationCompat.Builder? = null

    override fun onCreate() {
        super.onCreate()
        Log.d(sTAG, "onCreate")
        App.logger.putt("SdmClientForegroundService onCreate\n")
    }

    override fun onStartCommand(intent: Intent, flags: Int, startId: Int): Int {
        var sAction: String? = "null"
        Log.d(sTAG, "onStartCommand")
        App.logger.putt("SdmClientForegroundService onStartCommand\n")
        if (intent != null) {
            sAction = intent.action
            Log.d(sTAG,
                String.format("onStartCommand:%s  flags:%d  startId:%d", sAction, flags, startId)
            )
            if (sAction == STARTFOREGROUND_ACTION) {
                App.logger.putt("SdmClientForegroundService STARTFOREGROUND_ACTION\n")
                Log.d(sTAG, "onStartCommand. STARTFOREGROUND_ACTION")
                initapp()
            } else if (sAction == STOPFOREGROUND_ACTION) {
                App.logger.putt("SdmClientForegroundService STOPFOREGROUND_ACTION\n")
                Log.d(sTAG, "onStartCommand. STOPFOREGROUND_ACTION")
                stopForeground(true)
                val localBroadcastManager = LocalBroadcastManager
                    .getInstance(this)
                localBroadcastManager.sendBroadcast(
                    Intent(
                        "com.serialio.closeapp"
                    )
                )
                releaseResources()
                stopForeground(true)
                stopSelf()

            }
        } else Log.d(
            sTAG,
            String.format("onStartCommand:%s  flags:%d  startId:%d", sAction, flags, startId)
        )
        return START_REDELIVER_INTENT
    }

    override fun onTaskRemoved(rootIntent: Intent?) {
        println("onTaskRemoved called")
        super.onTaskRemoved(rootIntent)
        releaseResources()
        this.stopSelf()
    }

    override fun onDestroy() {
        App.logger.putt("SdmClientForegroundService onDestroy\n")
        Log.d(sTAG, "onDestroy")
        releaseResources()
        super.onDestroy()
    }

    private fun releaseResources() {
        Log.d(sTAG, "releaseResources")
        App.logger.putt("SdmClientForegroundService releaseResources\n")
    }

    fun initapp() {
        App.logger.putt("SdmClientForegroundService initapp\n")
        notificationBuilder = createForegroundNotificationBuilder(this, 1)
        startForeground(1, (notificationBuilder as NotificationCompat.Builder).build())
    }

    override fun onBind(intent: Intent): IBinder {
        return mBinder
    }

    class LocalBinder : Binder() {
        fun getServerInstance(): SdmClientForegroundService? {
            return SdmClientForegroundService()
        }
    }

}
