package com.serialio.sdm_tcpip_client.utils

import android.content.Context
import android.os.Build
import com.serialio.sdm_tcpip_client.App
import com.serialio.sdm_tcpip_client.BuildConfig
import com.serialio.sdm_tcpip_client.R
import com.serialio.sdm_tcpip_client.cih.ApiConfig
import com.serialio.sdm_tcpip_client.cih.RetrofitConfig
import okhttp3.MediaType
import okhttp3.RequestBody
import okhttp3.ResponseBody
import retrofit2.Call
import retrofit2.Callback
import java.io.File
import java.nio.file.Files
import java.text.SimpleDateFormat
import java.util.*

fun activateLogging(logPath: String) {
   //App.logger = LoggerSinglton.getInstance()
    if (!App.logger.isOpened()) {
        // Logger.gLogger.setFileLengthLimit(logging_size);
        if (App.logger.open(logPath, true)) {
            App.logger.putt("=== Start log ===\n")
            //Logger.gLogger.putt("App version %s\n", Constants.VER_INFO);
            val buildDate: Date = Date(BuildConfig.TIMESTAMP)
            val sBuildDate = SimpleDateFormat.getInstance().format(buildDate)
            var s = "\nApplication-infos:"
            s += "\n    OS Version: ${System.getProperty("os.version")}(${Build.VERSION.INCREMENTAL})"
            s += "\n    OS API Level: ${Build.VERSION.SDK_INT}"
            s += "\n    Device: ${Build.DEVICE}"
            s += "\n    Model (and Product): ${Build.MODEL} (${Build.PRODUCT})"
            s += "\n    BuildDate: $sBuildDate\n"
            App.logger.putt(s)
        }
    }
    App.logger.setFileLengthLimit(10000)
}
 fun uploadLogFile(context: Context, logPath: String, clientId: String,  fu: (String) -> Unit) {
    var result = "Failed"
    val Cal = Calendar.getInstance()
    val strTime = String.format("%tY%tm%td_%tH%tM%tS", Cal, Cal, Cal, Cal, Cal, Cal)
    var strZipname = java.lang.String.format(
        "%s/%s%s_%s_%s",
        context.getExternalFilesDir(null)?.getAbsolutePath(),
        "",
        clientId,
        strTime,
        "SioSDMClientLog.txt"
    )
    strZipname = strZipname.replace(".txt", ".zip")
    val strFiles: ArrayList<String?> = ArrayList<String?>()

    var sInfo = String.format("\n\nDevice Model: %s", Build.MODEL)
    sInfo += String.format(Locale.US, "\nSDK: %d", Build.VERSION.SDK_INT)
    sInfo += java.lang.String.format("\n ver: %s", BuildConfig.VERSION_NAME)
    sInfo += java.lang.String.format(
        "\nBuild date: %s\n",
        SimpleDateFormat("yyyy/MM/dd hh:mm:ss.SSS", Locale.getDefault())
            .format(BuildConfig.TIMESTAMP)
    ) //.UPDATE_DATE.toString())
     App.logger.putt(sInfo)


    val logFile = File(logPath)
    if (logFile.exists())  strFiles.add(logPath)
    val logcatPath: String = context.getExternalFilesDir(null)?.absolutePath + File.separator + "Logcat.txt"
    LogcatWriter.doWriteLogcat(logcatPath, context)
    strFiles.add(logcatPath)

     val prefPath: String = context.getExternalFilesDir(null)?.absolutePath + File.separator + "Preference.txt"
     val SharedPrefFile = File(prefPath)
     FileUtils.saveSharedPreferencesToFile(context, SharedPrefFile)
     strFiles.add(prefPath)

    Zipper.addToZip(strFiles.toTypedArray(), strZipname, strTime)

    val logcat = File(logcatPath)
    if (logcat.exists()) logcat.delete()
    val sName: String = context.getString(R.string.app_name)
    val appVersion = String.format(
        "%s Ver:%s Build:%s",
        sName,
        BuildConfig.VERSION_NAME,
        BuildConfig.VERSION_CODE.toString()
    )
    val login = "Log"
    val pw = "poi"
    val device = "Android"
    val user = "SioSdmClientApp"; //"user"
    // Map is used to multipart the file using okhttp3.RequestBody
    val logPath: String = strZipname
    val file = File(logPath)
    val fileName = file.name
    val length = file.length().toString()
    // Parsing any Media type file
    val requestBody: RequestBody =
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            RequestBody.create(MediaType.parse("*/*"), Files.readAllBytes(file.toPath()))
        } else {
            TODO("VERSION.SDK_INT < O")
        }

    val getResponse: ApiConfig? = RetrofitConfig.getRetrofit()?.create(ApiConfig::class.java)
    getResponse?.uploadFile(
        Base64Coder.encodeString(login),
        Base64Coder.encodeString(pw),
        Base64Coder.encodeString(fileName),
        length,
        Base64Coder.encodeString(device),
        Base64Coder.encodeString(user),
        Base64Coder.encodeString(appVersion),
        requestBody
    )?.enqueue(
        object : Callback<ResponseBody> {
            override fun onFailure(call: Call<ResponseBody>?, t: Throwable?) {
                // failure
                // TODO Auto-generated catch block
                App.logger.putt("Can't upload log!\n")
                fu(result + " Can't upload log")
            }

            override fun onResponse(
                call: Call<ResponseBody>,
                response: retrofit2.Response<ResponseBody>
            ) {
                // success
                if (response.isSuccessful()) {
                    val b: ResponseBody? = response.body()
                    if (b != null) {
                        result = parseCihResponse(b.byteStream(), b.contentLength().toInt())
                    }
                } else {
                    App.logger.putt("Can't upload log! Response not success\n")
                }
                fu(result)
            }
        })
}
